<?php

namespace DeubertIt\Debug\Command;

use DeubertIt\Debug\Util\ConfigUtil;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;

class ConfigValidator extends Command
{
    const LEVEL_SUCCESS = 0;
    const LEVEL_WARNING = 1;
    const LEVEL_ERROR = 2;

    private $level = self::LEVEL_SUCCESS;
    private $finalLevel = self::LEVEL_SUCCESS;

    /**
     * @var SymfonyStyle
     */
    private $io;

    /**
     * @var ConfigUtil
     */
    protected $configUtil;

    protected $filesystemCacheAdapter = [
        "Symfony\Component\Cache\Adapter\FilesystemAdapter"
    ];

    public function __construct(string $name, ParameterBagInterface $params, ConfigUtil $configUtil)
    {
        parent::__construct($name);

        $this->configUtil = $configUtil;
    }

    protected function configure()
    {
        $this
            ->addOption('only', null, InputOption::VALUE_REQUIRED, 'Validate only: folders,session,cache')
            ->setDescription('Validate symfony settings')
            ->setHelp('Validate settings and return success (0) / warning (1) / error (2) status.')
        ;
    }

    public function execute(InputInterface $input, OutputInterface $output)
    {
        $this->io = new SymfonyStyle($input, $output);

        $validate = explode(',', $input->getOption("only") ?? 'folders,session,cache');

        in_array('folders', $validate) && $this->checkFolders();
        in_array('session', $validate) && $this->checkSession();
        in_array('cache', $validate) && $this->checkCache();

        return $this->finalLevel;
    }

    public function checkFolders(): void
    {
        $this->prepare();
        $extra = $this->configUtil->getComposerExtra();

        if (!isset($extra['folders'])) {
            $this->report(self::LEVEL_WARNING, "composer.json:extra:folders is not defined and will be skipped");
            return;
        }

        foreach ($extra['folders'] as $key => $value) {
            $path = $this->configUtil->normalizePath($key);

            if (!is_writable($path)) {
                $this->report(self::LEVEL_ERROR, "$path is not writable!");
            }
        }

        $this->printSuccess("No filesystem issues found");
    }

    public function checkSession()
    {
        $this->prepare();

        $params = $this->configUtil->getAllParams();
        $handler = $this->configUtil->getContainerAlias("session.handler");

        if ($handler && ($handler == 'session.handler.native_file' || $handler == 'session.handler')) {
            $savePath = $params['session.save_path'];

            $this->report(self::LEVEL_WARNING, "Prefer Redis or Memcached to store sessions");

            if (!in_array($savePath, $this->configUtil->getSharedFolders())) {
                $this->report(self::LEVEL_ERROR, "Sessions save path is not shared: $savePath");
            }
        }

        $this->printSuccess("No session issues found");
    }

    public function checkCache()
    {
        $this->prepare();

        $cacheClass = $this->configUtil->getContainerClass("cache.app.recorder_inner");
        $doctrineClass = $this->configUtil->getContainerClass("doctrine.result_cache_pool.recorder_inner");

        if ($doctrineClass && in_array($cacheClass, $this->filesystemCacheAdapter)) {
            $this->report(self::LEVEL_ERROR, "It's not recommended to use $cacheClass for doctrine cache. Prefer Redis or Memcached adapter");
        }

        if (!$cacheClass || in_array($cacheClass, $this->filesystemCacheAdapter)) {
            $this->report(self::LEVEL_WARNING, "It's not recommended to use $cacheClass for app cache. Prefer Redis or Memcached adapter");
        }

        $this->printSuccess("No cache issues found");
    }

    private function report($level, $message)
    {
        if ($level == self::LEVEL_WARNING) {
            $this->io->warning($message);
        } else {
            $this->io->error($message);
        }

        if ($level > $this->finalLevel) {
            $this->finalLevel = $level;
        }

        if ($level > $this->level) {
            $this->level = $level;
        }
    }

    private function prepare()
    {
        $this->level = 0;
    }

    private function printSuccess($message)
    {
        if (!$this->level) {
            $this->io->success($message);
        }
    }
}
