<?php

namespace DeubertIt\Debug\Command;

use Composer\Semver\Semver;
use DeubertIt\Debug\Util\ConfigUtil;
use DeubertIt\Debug\Util\VersionParserUtil;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Throwable;

class Eol extends Command
{
    const PHP_OK_MESSAGE = "PHP version is OK";
    const PHP_NOT_SUPPORTED_MESSAGE = "PHP version is not supported anymore. Please, consider to upgrade";
    const PHP_SECURITY_MESSAGE = "PHP version is supported for critical security issues only";

    const SYMFONY_OK_MESSAGE = "Symfony version is OK";

    /**
     * @var SymfonyStyle
     */
    private $io;

    /**
     * @var ConfigUtil
     */
    private $composerUtil;

    /**
     * @var VersionParserUtil
     */
    private $versionParserUtil;

    /**
     * @var int
     */
    private $code = 0;

    /**
     * @var string
     */
    private $path;

    public function __construct(string $name, ConfigUtil $configUtil, VersionParserUtil $versionParserUtil)
    {
        parent::__construct($name);

        $this->composerUtil = $configUtil;
        $this->versionParserUtil = $versionParserUtil;
    }

    protected function configure()
    {
        $this
            ->addOption('project', null, InputOption::VALUE_REQUIRED, 'Path to project dir that contains composer.json')
            ->setDescription('Check PHP and Symfony EOL')
            ->setHelp('Print errors / warning if version is outdated')
        ;
    }

    public function execute(InputInterface $input, OutputInterface $output)
    {
        $this->path = $input->getOption("project") ?? getcwd();

        $this->io = new SymfonyStyle($input, $output);
        $this->checkSymfony();
        $this->checkPhp();

        return $this->code;
    }

    public function checkSymfony()
    {
        try {
            $supportedVersions = $this->versionParserUtil->parseSymfonySupportedVersions();
            $usedVersions = $this->getSymfonyVersions();

            $foundUnsupportedVersions = false;

            foreach ($usedVersions as $package => $version) {
                if (!$this->isSupportedVersion($version, $supportedVersions))  {
                    $this->io->error("Symfony version ($version) is not supported anymore. Please, consider to upgrade");
                    $foundUnsupportedVersions = true;
                }
            }

            if (!$foundUnsupportedVersions) {
                $this->io->success(self::SYMFONY_OK_MESSAGE);
            } else {
                $this->setStatusCode(2);
            }
        } catch (Throwable $e) {
            $this->io->error("Couldn't parse symfony eof versions");
        }
    }

    public function checkPhp()
    {
        $composer = $this->composerUtil->getComposerLockJson($this->path);
        $version = phpversion();

        if (!empty($composer['platform']) && !empty($composer['platform']['php'])) {
            $version = $composer['platform']['php'];
        }

        preg_match('/\d+.\d+/', $version, $formattedVersion);
        $version = reset($formattedVersion);

        try {
            $versions = $this->versionParserUtil->parsePhpVersions();

            if (!array_key_exists($version, $versions)) {
                $this->io->error(self::PHP_NOT_SUPPORTED_MESSAGE);
                return;
            }

            switch ($versions[$version]) {
                case "stable":
                    $this->io->success(self::PHP_OK_MESSAGE);
                    break;
                case "security":
                    $this->io->warning(self::PHP_SECURITY_MESSAGE);
                    $this->setStatusCode(1);
                    break;
                case "eol":
                    $this->io->error(self::PHP_NOT_SUPPORTED_MESSAGE);
                    $this->setStatusCode(2);
                    break;
            }
        } catch (Throwable $e) {
            $this->io->error("Couldn't parse PHP versions");
        }
    }

    private function isSupportedVersion($version, $versions)
    {
        preg_match('/(\d+.\d+).[0-9x]+/', $version, $formattedVersion);

        if (in_array($formattedVersion[1], $versions)) {
            return true;
        }

        return false;
    }

    private function getSymfonyVersions()
    {
        $lock = $this->composerUtil->getComposerLockJson($this->path);
        $versions = [];
        $packages = ["symfony/http-kernel"];

        foreach ($lock['packages'] as $package) {
            if (in_array($package['name'], $packages)) {
                $versions[$package['name']] = $package['version'];
            }
        }

        return $versions;
    }

    private function setStatusCode($code)
    {
        if ($code > $this->code) {
            $this->code = $code;
        }
    }
}
