<?php

namespace DeubertIt\Debug\Command;

use Symfony\Bridge\Twig\Mime\TemplatedEmail;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBagInterface;
use Symfony\Component\Mailer\MailerInterface;
use Symfony\Component\Mime\Address;
use Symfony\Component\Validator\Constraints as Assert;
use Symfony\Component\Validator\Validator\ValidatorInterface;
use Symfony\Component\Validator\ConstraintViolationList;
use Symfony\Component\Validator\ConstraintViolation;

/**
 * Class Mail
 * @package DeubertIt\Debug\Command
 */
class Mail extends Command
{
    /**
     * @var ParameterBagInterface
     */
    protected $params;

    /**
     * @var ValidatorInterface
     */
    protected $validator;

    /**
     * @var MailerInterface
     */
    protected $mailer;

    /**
     * Mail constructor.
     * @param string $name
     * @param ParameterBagInterface $params
     * @param ValidatorInterface $validator
     */
    public function __construct(string $name, ParameterBagInterface $params, ValidatorInterface $validator, MailerInterface $mailer)
    {
        parent::__construct($name);
        $this->params = $params;
        $this->validator = $validator;
        $this->mailer = $mailer;
    }

    protected function configure()
    {
        $this
            ->setDescription('Send email')
            ->setHelp('Send email configuration to given email address.')
            ->addArgument('email', InputArgument::REQUIRED, 'The email address')
        ;
    }

    /**
     * @param InputInterface $input
     * @param OutputInterface $output
     * @return int
     */
    public function execute(InputInterface $input, OutputInterface $output)
    {
        $container = $this->compileContainer();

        $mailerConfig = null;
        if ($container->hasExtension('framework')) {
            $frameworkExtensionConfig = $container->getExtensionConfig('framework');
            if (is_array($frameworkExtensionConfig) && !empty($frameworkExtensionConfig)) {
                foreach ($frameworkExtensionConfig as $extemsionConfig) {
                    if (isset($extemsionConfig['mailer'])) {
                        $mailerConfig = $extemsionConfig['mailer'];
                        break;
                    }
                }
            }
        }

        $swiftMailerConfig = null;
        if ($container->hasExtension('swiftmailer')) {
            $swiftMailerConfig = $container->getExtensionConfig('swiftmailer');
        }

        $io = new SymfonyStyle($input, $output);

        $debugEmail = $input->getArgument('email');
        $emailConstraint = new Assert\Email();
        $emailConstraint->message = 'Invalid email address';

        /** @var ConstraintViolationList $errors */
        $errors = $this->validator->validate($debugEmail, $emailConstraint);
        if (count($errors)) {
            /** @var ConstraintViolation $error */
            foreach ($errors as $error) {
                $io->error($error->getMessage());
            }
            return 0;
        }

        $mailerDsn = isset($_ENV['MAILER_DSN']) ? $_ENV['MAILER_DSN'] : null; // Mailer (default after 4.3)
        $mailerUrl = isset($_ENV['MAILER_URL']) ? $_ENV['MAILER_URL'] : null; // Swift mailer

        try {
            $context = [
                'default' => [
                    'env' => $mailerDsn,
                    'config' => $mailerConfig,
                ],
                'swiftmailer' => [
                    'env' => $mailerUrl,
                    'config' => $swiftMailerConfig,
                ]
            ];
            $this->sendEmail($debugEmail, $context);

            $io->success('Sent email to <'.$debugEmail.'>');
        } catch (Exception $e) {
            $io->error($e->getMessage());
        }

        return 0;
    }

    /**
     * @return ContainerBuilder
     * @throws \ReflectionException
     */
    private function compileContainer()
    {
        $kernel = clone $this->getApplication()->getKernel();
        $kernel->boot();

        $method = new \ReflectionMethod($kernel, 'buildContainer');
        $method->setAccessible(true);
        $container = $method->invoke($kernel);
        $container->getCompiler()->compile($container);

        return $container;
    }

    private function sendEmail(string $email, array $context = [])
    {
        $mail = new TemplatedEmail();
        $from = new Address($email, 'Symfony-dit-debug');

        $mail->from($from)
            ->to($email)
            ->subject('Symfony DIT debug email')
            ->htmlTemplate($this->params->get('dit_debug.template_mailer_config'))
            ->context($context)
        ;

        $this->mailer->send($mail);
    }
}
