<?php
namespace DeubertIt\Debug\Tests\Command;

use DeubertIt\Debug\Command\ConfigValidator;
use DeubertIt\Debug\Util\ConfigUtil;
use PHPUnit\Framework\TestCase;
use Symfony\Component\Console\Application;
use Symfony\Component\Console\Tester\CommandTester;
use Symfony\Component\DependencyInjection\ParameterBag\ParameterBag;

class ConfigValidatorTest extends TestCase
{
    /** @var ParameterBag|mixed */
    private $parameterBagMock;

    /** @var ConfigUtil|mixed */
    private $configUtilMock;

    /** @var CommandTester */
    private $commandTester;

    protected function setUp(): void
    {
        $this->parameterBagMock = $this->getMockBuilder(ParameterBag::class)
            ->disableOriginalConstructor()
            ->getMock();

        $this->configUtilMock = $this->getMockBuilder(ConfigUtil::class)
            ->disableOriginalConstructor()
            ->getMock();

        $application = new Application();
        $application->add(new ConfigValidator('_', $this->parameterBagMock, $this->configUtilMock));
        $command = $application->find('_');
        $this->commandTester = new CommandTester($command);
    }

    protected function tearDown(): void
    {
        $this->parameterBagMock = null;
        $this->configUtilMock = null;
        $this->commandTester = null;
    }

    public function testNoFolders()
    {
        $this->configUtilMock
            ->method("getComposerExtra")
            ->willReturn([]);

        $this->commandTester->execute(['--only' => 'folders']);

        $this->assertEquals(ConfigValidator::LEVEL_WARNING, $this->commandTester->getStatusCode());
        $this->assertStringContainsString('folders is not defined', $this->commandTester->getDisplay());
    }

    public function testSession()
    {
        $dataSet = [
            [
                'input' => [ 'session' => 'session.handler.native_file' ],
                'expect' => [
                    'status' => ConfigValidator::LEVEL_ERROR,
                    'message' => 'Sessions save path is not shared'
                ]
            ],
            [
                'input' => [
                    'shared_folders' => ['/test/folder'],
                    'save_path' => '/test/folder',
                    'session' => 'session.handler.native_file'
                ],
                'expect' => [ 'status' => ConfigValidator::LEVEL_WARNING ]
            ],
            [
                'input' => [
                    'session' => 'session.handler.redis'
                ],
                'expect' => [ 'status' => ConfigValidator::LEVEL_SUCCESS ]
            ]
        ];

        foreach ($dataSet as $data) {
            $this->setUp();

            $input = $data['input'];
            $expect = $data['expect'];

            $this->configUtilMock
                ->method('getContainerAlias')
                ->willReturn($input['session']);

            $this->configUtilMock
                ->method('getSharedFolders')
                ->willReturn($input['shared_folders'] ?? []);

            $this->configUtilMock
                ->method('getAllParams')
                ->willReturn([
                    'session.save_path' => $input['save_path'] ?? '/test/path'
                ]);

            $this->configUtilMock
                ->method("getComposerExtra")
                ->willReturn([
                    'folders' => $input['extra_folders'] ?? []
                ]);

            $this->commandTester->execute(['--only' => 'session']);

            $this->assertEquals($expect['status'], $this->commandTester->getStatusCode());

            if (!empty($expect['message'])) {
                $this->assertStringContainsString($expect['message'], $this->commandTester->getDisplay());
            }

            $this->tearDown();
        }
    }

    public function testPermissionsFail()
    {
        $dir1 = 'dir_1';
        $dir2 = 'dir_2';

        $this->configUtilMock
            ->method('normalizePath')
            ->willReturnCallback(function ($path) {
                return $path;
            });

        $this->configUtilMock
            ->method("getComposerExtra")
            ->willReturn([
                'folders' => [
                    $dir1 => [ 'shared' => true ],
                    $dir2 => [ 'writable' => true ],
                ]
            ]);

        mkdir($dir1, 0444);
        mkdir($dir2);

        $this->commandTester->execute(['--only' => 'folders']);

        $this->assertEquals(ConfigValidator::LEVEL_ERROR, $this->commandTester->getStatusCode());
        $this->assertStringContainsString("$dir1 is not writable", $this->commandTester->getDisplay());
        $this->assertStringNotContainsString("$dir2 is not writable", $this->commandTester->getDisplay());

        rmdir($dir1);
        rmdir($dir2);
    }
}